import 'dart:async';

import 'package:isolate_bloc/isolate_bloc.dart';

import 'isolate_bloc.dart';

typedef IsolateBlocCreator<Event, State> = IsolateBloc<Event, State> Function();

/// Maintain [IsolateBloc]s in isolate
class IsolateBlocConnector {
  static final IsolateBlocConnector instance = IsolateBlocConnector._();
  final _initialStates = <Type, Object>{};
  final _freeBlocs = <Type, IsolateBloc>{};
  final _blocCreators = <Type, IsolateBlocCreator>{};
  final _createdBlocs = <String, IsolateBloc>{};
  final _createdBlocsByType = <Type, IsolateBloc>{};
  final _initializeCompleter = Completer();

  Map<Type, Object> get initialStates => _initialStates;

  void initializeCompleted() => _initializeCompleter.complete();

  IsolateBlocConnector._();

  /// Returns new bloc from cached in [_freeBlocs] or create new one.
  IsolateBloc getFreeBlocByType(Type type) {
    IsolateBloc bloc;
    if (_freeBlocs.containsKey(type)) {
      bloc = _freeBlocs.remove(type);
    } else {
      bloc = _blocCreators[type]();
    }
    _createdBlocs[bloc.id] = bloc;
    _createdBlocsByType[type] = bloc;
    return bloc;
  }

  /// Wait when every bloc registered with [register] func and return [IsolateBloc]
  Future<T> getBloc<T extends IsolateBloc>() async {
    await _initializeCompleter.future;
    if (_createdBlocsByType.containsKey(T)) {
      return _createdBlocsByType[T];
    } else {
      return _freeBlocs[T] = _blocCreators[T]();
    }
  }

  IsolateBlocWrapper<State> getBlocWrapper<Bloc extends IsolateBloc, State>() {
    IsolateBlocWrapper<State> wrapper;
    Bloc isolateBloc;
    getBloc<Bloc>().then((bloc) {
      isolateBloc = bloc;
      bloc.listen(wrapper.stateReceiver);
      wrapper.setUuid(bloc.id);
    });
    final onBLocClose = (_) => isolateBloc?.close();
    final eventReceiver = (Object event) => isolateBloc?.add(event);
    wrapper = IsolateBlocWrapper.noInitState(eventReceiver, onBLocClose);
    return wrapper;
  }

  IsolateBloc getBlocById(String id) {
    return _createdBlocs[id];
  }

  IsolateBloc removeBloc(String id) {
    return _createdBlocsByType.remove(_createdBlocs.remove(id));
  }

  void register<Event, State>(IsolateBlocCreator<Event, State> creator) {
    // ignore: close_sinks
    final bloc = creator();
    _initialStates[bloc.runtimeType] = bloc.state;
    _freeBlocs[bloc.runtimeType] = bloc;
    _blocCreators[bloc.runtimeType] = creator;
  }
}
