import 'isolate_bloc.dart';
import 'isolate_bloc_connector.dart';
import 'service_events.dart';

/// Listen for [ServiceEvent]s in isolate
class IsolateServiceEventListener {
  final _blocConnector = IsolateBlocConnector.instance;
  final void Function(ServiceEvent) _sendEvent;
  final Stream<ServiceEvent> _eventsStream;

  IsolateServiceEventListener(this._sendEvent, this._eventsStream) {
    _eventsStream.listen(listener);
  }

  void listener(ServiceEvent event) {
    if (event is IsolateBlocTransitionEvent) {
      _blocConnector.getBlocById(event.blocUuid).add(event.event);
    } else if (event is CreateIsolateBlocEvent) {
      var bloc = _blocConnector.getFreeBlocByType(event.blocType);
      _connectBloc(bloc);
      _sendEvent(IsolateBlocCreatedEvent(bloc.runtimeType, bloc.id));
    } else if (event is CloseIsolateBlocEvent) {
      _blocConnector.removeBloc(event.blocUuid).close();
    }
  }

  void _connectBloc(IsolateBloc bloc) {
    bloc.listen(
      (state) => _sendEvent(
        IsolateBlocTransitionEvent(bloc.id, state),
      ),
    );
  }
}
