import 'dart:async';

import 'bloc_manager.dart';

import 'service_events.dart';

/// Listen for [ServiceEvent]s from isolate
class ServiceEventListener {
  final void Function(ServiceEvent) sendEvent;
  final Stream<ServiceEvent> _eventsStream;
  StreamSubscription<ServiceEvent> _eventSubscription;
  final _initializeCompleter = Completer<Map<Type, Object>>();

  ServiceEventListener(this.sendEvent, this._eventsStream) {
    _eventSubscription = _eventsStream.listen(listener);
  }

  Future<Map<Type, Object>> getInitialStates() async {
    return _initializeCompleter.future;
  }

  void listener(ServiceEvent event) {
    if (event is IsolateBlocsInitialized) {
      _initializeCompleter.complete(event.initialStates);
    } else if (event is IsolateBlocCreatedEvent) {
      BlocManager.instance.bindFreeWrapper(event.blocType, event.blocUuid);
    } else if (event is IsolateBlocTransitionEvent) {
      BlocManager.instance
          .getWrapper(event.blocUuid)
          .stateReceiver(event.event);
    }
  }

  void dispose() {
    _eventSubscription?.cancel();
  }
}
