import 'package:flutter_test/flutter_test.dart';
import 'package:isolate_bloc/isolate_bloc.dart';

import 'blocs.dart';
import 'initializers.dart';

void main() {
  group("Full bloc's infrastructure test", () {
    test("Test simple operations like register, create, add and listen.",
        () async {
      final states = [0, -1];
      await initialize(Initializers.counterTest, reCreate: true);
      // ignore: close_sinks
      var counterBloc = createBloc<CounterBloc, int>();
      counterBloc.listen((state) {
        expect(
          counterBloc.state,
          0,
          reason: "counter bloc have unexpected state",
        );
        states.removeAt(0);
      });
      counterBloc.add(CountEvent.decrement);
    });

    test("Test `getBloc` and `getBlocWrapper`", () async {
      final states = [0, -1, 0, 1, 2, 3];
      final wrapperHistory = <int>[];
      final blocHistory = <int>[];
      await initialize(Initializers.injectionTest, reCreate: true);
      // ignore: close_sinks
      var counterBloc = createBloc<CounterBloc, int>();
      // ignore: close_sinks
      var historyWithBlocInjection =
          createBloc<CounterHistoryBlocInjector, List<int>>();
      // ignore: close_sinks
      var historyWithWrapperInjection =
          createBloc<CounterHistoryWrapperInjector, List<int>>();

      historyWithBlocInjection.listen((blocState) {
        if (blocState.isNotEmpty) {
          blocHistory.add(states[blocState.length - 1]);
        }
        expect(
          blocState,
          blocHistory,
          reason:
              "CounterHistoryBloc with getBloc injector have unexpected state.",
        );
      });
      historyWithWrapperInjection.listen((wrapperState) {
        if (wrapperState.isNotEmpty) {
          wrapperHistory.add(states[wrapperState.length - 1]);
        }
        expect(
          wrapperState,
          wrapperHistory,
          reason: "CounterHistoryBloc with getBlocWrapper injector "
              "have unexpected state.",
        );
      });

      counterBloc.add(CountEvent.decrement);
      counterBloc.add(CountEvent.increment);
      counterBloc.add(CountEvent.increment);
      counterBloc.add(CountEvent.increment);
      counterBloc.add(CountEvent.increment);
    });

    test("Test implicit bloc creation with `getBloc`", () async {
      /// CounterIncrementerBloc create bloc by calling getBloc and add increment event.
      /// createBloc<CounterBloc, int> must provide bloc which was created by CounterIncrementBloc
      /// and this bloc must have the same states.
      await initialize(Initializers.injectionTest, reCreate: true);
      createBloc<CounterIncrementerBloc, int>();
      final firstIncrementHistory = [0, 1];
      createBloc<CounterBloc, int>().listen((state) {
        expect(state, firstIncrementHistory.removeAt(0), reason: "");
      });
      createBloc<CounterIncrementerBloc, int>();
      final secondIncrementHistory = [0, 1, 2];
      createBloc<CounterBloc, int>().listen((state) {
        expect(state, secondIncrementHistory.removeAt(0), reason: "");
      });
    });
  });
}
